/**
 * @file buzzer_pwm.c
 * @author
 * @brief
 * @version 0.1
 * @date 2023-06-09
 *
 * @copyright Copyright (c) 2023
 *
 */

/* Includes --------------------------------------------------------------------------------------*/
#include <nrfx_pwm.h>
#include <hal/nrf_gpio.h>
#include "buzzer_pwm.h"
#include "app_common.h"

/* Private macros --------------------------------------------------------------------------------*/
// This is for tracking PWM instances being used, so we can uninitialized only
// the relevant ones when switching from one demo to another.
#define USED_PWM(idx) (1UL << idx)

/* Private param ---------------------------------------------------------------------------------*/
static nrfx_pwm_t BUZZ_PWM = NRFX_PWM_INSTANCE(2);

/* Beep tone used to beep when search is requested */
static nrf_pwm_values_wave_form_t tone_4k_values[] = {
  /* chn_0          chn_1          chnl_2   counter top */
    {150,  (150 | NRF_PWM_INVERT),  150,        250},
    {0,     0, 					    0, 	        250},
    {200,  (200 | NRF_PWM_INVERT),  200,        400},
    {0, 	0, 					    0, 	        250},
    {150,  (150 | NRF_PWM_INVERT),  150,        250},
    {0, 	0, 					    0, 	        250},
    {200,  (200 | NRF_PWM_INVERT),  200,        400},
    {0, 	0, 					    0, 	        250},
    {150,  (150 | NRF_PWM_INVERT),  150,        250},
    {0, 	0, 					    0, 	        250},
    {200,  (200 | NRF_PWM_INVERT),  200,        400},
    {0, 	0, 					    0, 	        250},
    {150,  (150 | NRF_PWM_INVERT),  150,        250},
    {0, 	0, 					    0, 	        250},
    {150,  (150 | NRF_PWM_INVERT),  150,        250},
    {0, 	0, 					    0, 	        250},
    {150,  (150 | NRF_PWM_INVERT),  150,        250},
   {0, 	0, 					    0, 	        250},
   {0, 	0, 					    0, 	        250},
    {0, 	0, 					    0, 	        250},
    {0, 	0, 					    0, 	        250},
    {0, 	0, 					    0, 	        250},
    {0, 	0, 					    0, 	        250},
    {0, 	0, 					    0, 	        250},
    {0, 	0, 					    0, 	        250},
    {0, 	0, 					    0, 	        250},
    {0, 	0, 					    0, 	        250},
};

/* Connection tone */
static nrf_pwm_values_wave_form_t slow_tone[] = {
  {37,  (37 | NRF_PWM_INVERT),  0,  250},
  {0,    0, 					0, 	250},
  {0,    0, 					0, 	250},
  {0,    0, 					0, 	250},
};

/* Connection tone */
static nrf_pwm_values_wave_form_t mid_tone[] = {
  {37,  (37 | NRF_PWM_INVERT),  0,  250},
  {0,    0, 					0, 	250},
  {0,    0, 					0, 	250},
};

/* Connection tone */
static nrf_pwm_values_wave_form_t fast_tone[] = {
  {37,  (37 | NRF_PWM_INVERT),  0,  250},
  {0,    0, 					0, 	250},
};

/* Global param ----------------------------------------------------------------------------------*/
nrfx_pwm_config_t const config0 =
{
    .output_pins =
    {
    	BZ_PWMA,                    // channel 0
    	BZ_PWMB,                    // channel 2
        NRF_PWM_PIN_NOT_CONNECTED,  // channel 1
        NRF_PWM_PIN_NOT_CONNECTED,  // channel 3
    },

    .irq_priority = 7,
    .base_clock   = NRF_PWM_CLK_1MHz,
    .count_mode   = NRF_PWM_MODE_UP,
    .top_value    = 250,
    .load_mode    = NRF_PWM_LOAD_WAVE_FORM,
    .step_mode    = NRF_PWM_STEP_AUTO
};

nrf_pwm_sequence_t seq;// =
// {
//     .values.p_wave_form  = tone_4k_values,
//     .length              = NRF_PWM_VALUES_LENGTH(tone_4k_values),
//     .repeats             = 200,
//     .end_delay           = 0
// };

void pwm_pin_set (void)
{
  // dont forget to rename as macro

  nrf_gpio_cfg_output(11);
  nrf_gpio_pin_set(11);

  nrf_gpio_cfg_output(14);
  nrf_gpio_pin_set(14);
}

/**
 * @brief Configure PWM for buzzer
 *
 */
void buzzer_init (void)
{
	uint8_t err = 0;
	uint32_t out_pins[NRF_PWM_CHANNEL_COUNT] = {
        BZ_PWMA, BZ_PWMB, NRF_PWM_PIN_NOT_CONNECTED, NRF_PWM_PIN_NOT_CONNECTED
    };

	pwm_pin_set();
    nrf_pwm_pins_set(BUZZ_PWM.p_reg, out_pins);
	err = nrfx_pwm_init(&BUZZ_PWM, &config0, NULL, NULL);
}

/**
 * @brief Function turn on buzzer for alert using PWm
 *
 */
void buzzer_start_alert_tone (void)
{
    seq.values.p_wave_form = tone_4k_values;
    seq.length             = NRF_PWM_VALUES_LENGTH(tone_4k_values);
    seq.repeats            = 200;
    seq.end_delay          = 10;
	nrfx_pwm_simple_playback(&BUZZ_PWM, &seq, 1, NRFX_PWM_FLAG_LOOP);
}

/**
 * @brief Function turn on buzzer for alert using PWm
 *
 */
void buzzer_start_homing_tone_slow (void)
{
    seq.values.p_wave_form = slow_tone;
    seq.length             = NRF_PWM_VALUES_LENGTH(slow_tone);
    seq.repeats            = 200;
    seq.end_delay          = 0;
	nrfx_pwm_simple_playback(&BUZZ_PWM, &seq, 1, NRFX_PWM_FLAG_LOOP);
}

/**
 * @brief Function turn on buzzer for alert using PWm
 *
 */
void buzzer_start_homing_tone_mid (void)
{
    seq.values.p_wave_form = mid_tone;
    seq.length             = NRF_PWM_VALUES_LENGTH(mid_tone);
    seq.repeats            = 200;
    seq.end_delay          = 0;
	nrfx_pwm_simple_playback(&BUZZ_PWM, &seq, 1, NRFX_PWM_FLAG_LOOP);
}

/**
 * @brief Function turn on buzzer for alert using PWm
 *
 */
void buzzer_start_homing_tone_fast (void)
{
    seq.values.p_wave_form = fast_tone;
    seq.length             = NRF_PWM_VALUES_LENGTH(fast_tone);
    seq.repeats            = 200;
    seq.end_delay          = 0;
	nrfx_pwm_simple_playback(&BUZZ_PWM, &seq, 1, NRFX_PWM_FLAG_LOOP);
}

/**
 * @brief Turn OFF buzzer PWM
 *
 */
void buzzer_tone_stop (void)
{
	nrfx_pwm_stop(&BUZZ_PWM, true);
    while(!nrfx_pwm_stopped_check(&BUZZ_PWM));
}
